import java.awt.Point;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Set;
import java.util.StringTokenizer;
import java.util.TreeSet;


/**
 * ICPC - CTU Open Contest 2021
 * Sample Solution: TomTom Cruise
 * 
 * @author Martin Kacer
 */
public class BlobsMK {
	StringTokenizer st = new StringTokenizer("");
	BufferedReader input = new BufferedReader(new InputStreamReader(System.in));
	boolean hasNextToken() {
		try {
			while (!st.hasMoreTokens()) {
				String line = input.readLine();
				if (line == null) return false;
				st = new StringTokenizer(line);
			}
		} catch (IOException ex) { throw new RuntimeException(ex); }
		return true;
	}
	String nextToken() {
		return (!hasNextToken()) ? null : st.nextToken();
	}
	int nextInt() {
		return Integer.parseInt(nextToken());
	}
	public static void main(String[] args) {
		new BlobsMK().run();
	}
	static char[] rep(char c, int n) { char[] r = new char[n]; Arrays.fill(r, c); return r; }
	char other(char c) { return c == '.' ? '#' : '.'; }
	
	int n, m;
	char[][] map;
	StringBuilder str = new StringBuilder();
	Set<Point> inner = new TreeSet<>((a,b) -> (a.x != b.x) ? a.x - b.x : a.y - b.y);
	
	void remove(int x, int y, char c) {
		if (map[x][y] == other(c)) inner.add(new Point(x, y));
		if (map[x][y] != c) return;
		map[x][y] = 0;
		if (x > 0) remove(x-1, y, c);
		if (x < n-1) remove(x+1, y, c);
		if (y > 0) remove(x, y-1, c);
		if (y < m-1) remove(x, y+1, c);
	}
	
	CharSequence read(int x, int y, char c) {
		inner.clear();
		StringBuilder res = new StringBuilder(" ");
		remove(x, y, c);
		List<Point> pts = new ArrayList<>(inner);
		int cnt = 0;
		char oc = other(c);
		for (Point p : pts)
			if (map[p.x][p.y] == oc) {
				++cnt;
				res.append(read(p.x, p.y, oc));
			}
		if (cnt == 0) return "";
		res.setCharAt(0, (char)('a'-1+cnt));
		return res;
	}
	
	void run() {
		n = nextInt() + 2; m = nextInt() + 2;
		map = new char[n][];
		map[0] = rep('.', m); map[n-1] = rep('.', m);
		for (int i = 1; i < n-1; ++i) map[i] = ("."+nextToken()+".").toCharArray();
		remove(0, 0, '.');
		Point p = inner.iterator().next();
		StringBuilder str = new StringBuilder(read(p.x, p.y, '#')).reverse();
		int rows = 49 + str.length() * 2, cols = str.length() * 4 + 1, r1 = 0, r2 = rows, c1 = 0, c2 = cols;
		System.out.println(rows + " " + cols);
		map = new char[rows][cols];
		char c = '#';
		for (char x : str.toString().toCharArray()) {
			for (int i = r1; i < r2; ++i) map[i][c1] = map[i][c1+1] = map[i][c1+2] = map[i][c2-1] = c;
			for (int j = c1; j < c2; ++j) map[r1][j] = map[r2-1][j] = c;
			c = other(c);
			for (int i = r1+1; x > 'a'; --x, i += 2)
				map[i][c1+1] = c;
			++r1; --r2; c1 += 3; --c2;
		}
		for (int i = r1; i < r2; ++i)
			for (int j = c1; j < c2; ++j) map[i][j] = c;
		for (int i = 0; i < rows; ++i)
			System.out.println(map[i]);
	}
}
